/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOgl
 *      Filename:       glu.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    SWIG file for wrapping the OpenGL utility library GLU
 *                      contained in file glu.h.
 *
 *****************************************************************************/

%{
#if defined (WIN32)
#include <windows.h>
#include <wingdi.h>
#endif
#include "GL/glu.h"
%}

%typemap(in) double * (double *tmp) {
    if ((tmp = GetDoubleList (interp, $input)) == NULL) {
        return TCL_ERROR;
    } else {
        $1 = tmp;
    }
}

%typemap(in) float * (float *tmp) {
    if ((tmp = GetFloatList (interp, $input)) == NULL) {
        return TCL_ERROR;
    } else {
        $1 = tmp;
    }
}

// Do not generate wrappper code for the GLU callback functions.
// This is work to be done.

%ignore gluQuadricCallback;
%ignore gluTessCallback;
%ignore gluNurbsCallback;

%include tcl3dDefines.i

#define WINGDIAPI
#define APIENTRY


// Redefine the following functions with a return value, so that the SWIG
// generated wrapper function returns a Tcl string.
// Maybe this could be made more elegant with typemaps ??

const char *gluGetString (GLenum name);
const char *gluErrorString (GLenum errorCode);

#if defined (WIN32)

int gluProject (
    GLdouble        objx, 
    GLdouble        objy, 
    GLdouble        objz,  
    const GLdouble  modelMatrix[16], 
    const GLdouble  projMatrix[16], 
    const GLint     viewport[4], 
    GLdouble        *OUTPUT, 
    GLdouble        *OUTPUT, 
    GLdouble        *OUTPUT);

int gluUnProject (
    GLdouble       winx, 
    GLdouble       winy, 
    GLdouble       winz, 
    const GLdouble modelMatrix[16], 
    const GLdouble projMatrix[16], 
    const GLint    viewport[4], 
    GLdouble       *OUTPUT, 
    GLdouble       *OUTPUT, 
    GLdouble       *OUTPUT);
#else

GLint gluProject (
    GLdouble objX, GLdouble objY, GLdouble objZ,
    const GLdouble *model, const GLdouble *proj, const GLint *view,
    GLdouble* OUTPUT, GLdouble* OUTPUT, GLdouble* OUTPUT);

GLint gluUnProject (
    GLdouble winX, GLdouble winY, GLdouble winZ,
    const GLdouble *model, const GLdouble *proj, const GLint *view,
    GLdouble* OUTPUT, GLdouble * OUTPUT, GLdouble* OUTPUT);

#endif

%include "GL/glu.h"
