/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOgl
 *      Filename:       tkphoto.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    SWIG file for wrapping Tk photo images.
 *
 *****************************************************************************/

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <tcl.h>
#include <tk.h>

static int tcl3dPhotoChans (Tcl_Interp *interp, char *photoName) 
{
    Tk_PhotoHandle photo;
    Tk_PhotoImageBlock photoCont;

    photo = Tk_FindPhoto(interp, photoName);
    if (!photo) {
        Tcl_AppendResult (interp, "cannot find photo image: ",
                          photoName, (char *)NULL);
        return 0;
    }
    Tk_PhotoGetImage(photo, &photoCont);
    return photoCont.pixelSize;
}

static int tcl3dPhotoToVector (Tcl_Interp *interp, char *photoName,
                              void *imgVector, int numChans,
                              float scale, float offset)
{
    Tk_PhotoHandle photo;
    Tk_PhotoImageBlock photoCont;
    int x, y, w, h, chansInImg, chansToSkip;
    unsigned char *srcPtr;
    unsigned char *imgVec = (unsigned char *) imgVector;

    photo = Tk_FindPhoto(interp, photoName);
    if (!photo) {
        Tcl_AppendResult (interp, "cannot find photo image: ",
                          photoName, (char *)NULL);
        return TCL_ERROR;
    }
    Tk_PhotoGetImage(photo, &photoCont);
    w = photoCont.width;
    h = photoCont.height;
    chansInImg = photoCont.pixelSize;

    srcPtr = photoCont.pixelPtr;

    if (numChans > chansInImg || numChans <= 0) {
        numChans = chansInImg;
    }
    chansToSkip = chansInImg - numChans;

    for (y = h-1; y >= 0; y--) {
        if (numChans == 1) {
            for (x = 0; x < w; x++) {
                imgVec[y*w+x] = (unsigned char) (*srcPtr++ * scale + offset);
                srcPtr += chansToSkip;
            }
        } else if (numChans == 2) {
            for (x = 0; x < w; x++) {
                imgVec[(y*w+x)*2 +0] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*2 +1] = (unsigned char) (*srcPtr++ * scale + offset);
                srcPtr += chansToSkip;
            }
        } else if (numChans == 3) {
            for (x = 0; x < w; x++) {
                imgVec[(y*w+x)*3 +0] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*3 +1] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*3 +2] = (unsigned char) (*srcPtr++ * scale + offset);
                srcPtr += chansToSkip;
            }
        } else {
            for (x = 0; x < w; x++) {
                imgVec[(y*w+x)*4 +0] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*4 +1] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*4 +2] = (unsigned char) (*srcPtr++ * scale + offset);
                imgVec[(y*w+x)*4 +3] = (unsigned char) (*srcPtr++ * scale + offset);
                srcPtr += chansToSkip;
            }
        }
    }
    return TCL_OK;
}

/* OBSOLETE tcl3dPhoto2Vector 0.3.2 tcl3dPhotoToVector */
static int tcl3dPhoto2Vector (Tcl_Interp *interp, char *photoName,
                              void *imgVector) 
{
    return tcl3dPhotoToVector (interp, photoName, imgVector, -1, 1.0, 0.0);
}

static int tcl3dVectorToPhoto (Tcl_Interp *interp, void *imgVector, 
                               const char *photoName, int width, int height,
                               int numChans)
{
    int y, usedChans;
    Tk_PhotoHandle photo;
    Tk_PhotoImageBlock photoCont;
    unsigned char *imgVec = (unsigned char *) imgVector;
    unsigned char *srcPtr, *dstPtr;
    unsigned char *srcStop;
    unsigned char *scanBuf;

    if (!(photo = Tk_FindPhoto (interp, (char *)photoName))) {
        Tcl_AppendResult (interp, "cannot find photo image: ",
                          photoName, (char *)NULL);
        return TCL_ERROR;
    }

    #if (TK_MAJOR_VERSION > 8) || \
       ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION >= 5))
        Tk_PhotoSetSize (interp, photo, width, height);
    #else
        Tk_PhotoSetSize (photo, width, height);
    #endif

    usedChans = (numChans <= 3? 3: 4);

    if (!(scanBuf = (unsigned char *)malloc (width * usedChans * sizeof (unsigned char)))) {
        Tcl_AppendResult (interp, "cannot alloc scanline bufer for photo: ",
                          photoName, (char *)NULL);
        return TCL_ERROR;
    }

    photoCont.pixelSize = usedChans;
    photoCont.pitch = width * usedChans;
    photoCont.width = width;
    photoCont.height = 1;
    photoCont.offset[0] = 0;
    photoCont.offset[1] = 1;
    photoCont.offset[2] = 2;
    photoCont.offset[3] = (usedChans == 3? 0: 3);

    photoCont.pixelPtr = scanBuf;
    srcPtr = imgVec;

    /* All channels are in UBYTE format. */
    for (y = 0; y < height; y++) {
        dstPtr = scanBuf;
        srcStop = srcPtr + numChans * width;
        if (numChans == 1) {
            while (srcPtr < srcStop) {
                *(dstPtr++) = *srcPtr;
                *(dstPtr++) = *srcPtr;
                *(dstPtr++) = *(srcPtr++);
            }
        } else if (numChans == 2) {
            while (srcPtr < srcStop) {
                *(dstPtr++) = *srcPtr;
                *(dstPtr++) = *srcPtr;
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
            }
        } else if (numChans == 3) {
            while (srcPtr < srcStop) {
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
            }
        } else {
            while (srcPtr < srcStop) {
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
                *(dstPtr++) = *(srcPtr++);
            }
        }
        #if (TK_MAJOR_VERSION > 8) || \
            ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION >= 5))
            Tk_PhotoPutBlock (interp, photo, &photoCont, 0, height -1 - y,
                              width, 1, TK_PHOTO_COMPOSITE_OVERLAY);
        #elif ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION > 3))
            Tk_PhotoPutBlock (photo, &photoCont, 0, height -1 - y,
                              width, 1, TK_PHOTO_COMPOSITE_OVERLAY);
        #else
            Tk_PhotoPutBlock (photo, &photoCont, 0, height -1 - y,
                              width, 1);
        #endif
    }

    free (scanBuf);
    return TCL_OK;
}

/* OBSOLETE tcl3dVector2Photo 0.3.3 tcl3dVectorToPhoto */
static int tcl3dVector2Photo (Tcl_Interp *interp, void *imgVector, 
                               const char *photoName, int width, int height,
                               int numChans)
{
    return tcl3dVectorToPhoto (interp, imgVector, photoName, width, height, numChans);
}

%}

int tcl3dPhotoChans    (Tcl_Interp *interp, char *imgName);
int tcl3dPhotoToVector (Tcl_Interp *interp, char *imgName, void *imgVec, 
                        int numChans, float scale, float offset);
int tcl3dPhoto2Vector  (Tcl_Interp *interp, char *imgName, void *imgVec);
int tcl3dVector2Photo  (Tcl_Interp *interp, void *imgVec, 
                        const char *photoName, int width, int height,
                        int numChans);
int tcl3dVectorToPhoto (Tcl_Interp *interp, void *imgVec, 
                        const char *photoName, int width, int height,
                        int numChans);
