/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on 
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOgl
 *      Filename:       tcl3dUtilPortable.h
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    Functions to convert integer and floating point
 *                      numbers between the processor's data format and
 *                      a machine-independent representation.
 *
 *****************************************************************************/

#ifndef __TCL3D_UTIL_PORTABLE_H__
#define __TCL3D_UTIL_PORTABLE_H__

typedef unsigned char  UInt8;
typedef unsigned short UInt16;
typedef unsigned int   UInt32;

typedef signed char  Int8;
typedef signed short Int16;
typedef signed int   Int32;

typedef float  Float32;
typedef double Float64;

typedef enum {
    UT_False = 0,
    UT_True  = 1
} UT_Bool;

#ifdef __cplusplus
   extern "C" {
#endif

    UT_Bool tcl3dIsIntel (void);

    void tcl3dInt8ToByte     ( Int8 x,   Int8 buf[1]);
    void tcl3dUInt8ToByte    (UInt8 x,   Int8 buf[1]);
    void tcl3dInt16ToByte    ( Int16 x,  Int8 buf[2]);
    void tcl3dUInt16ToByte   (UInt16 x,  Int8 buf[2]);
    void tcl3dInt32ToByte    ( Int32 x,  Int8 buf[4]);
    void tcl3dUInt32ToByte   (UInt32 x,  Int8 buf[4]);
    void tcl3dFloat32ToByte  (Float32 x, Int8 buf[4]);
    void tcl3dFloat32To3Byte (Float32 x, Int8 buf[3]);
    void tcl3dFloat32To2Byte (Float32 x, Int8 buf[2]);
    void tcl3dFloat64ToByte  (Float64 x, Int8 buf[8]);

    Int8    tcl3dByteToInt8     (const Int8 buf[1]);
    UInt8   tcl3dByteToUInt8    (const Int8 buf[1]);
    Int16   tcl3dByteToInt16    (const Int8 buf[2]);
    UInt16  tcl3dByteToUInt16   (const Int8 buf[2]);
    Int32   tcl3dByteToInt32    (const Int8 buf[4]);
    UInt32  tcl3dByteToUInt32   (const Int8 buf[4]);
    Float32 tcl3dByteToFloat32  (const Int8 buf[4]);
    Float32 tcl3d3ByteToFloat32 (const Int8 buf[3]);
    Float32 tcl3d2ByteToFloat32 (const Int8 buf[2]);
    Float64 tcl3dByteToFloat64  (const Int8 buf[8]);

#ifdef __cplusplus
   }
#endif

#endif /* __TCL3D_UTIL_PORTABLE_H__ */
