/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOgl
 *      Filename:       tcl3dVecMath.h
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    Declarations for OpenGL compatible 
 *                      vector and matrix functions.
 *
 *****************************************************************************/

#ifndef __TCL3D_VECMATH_H__
#define __TCL3D_VECMATH_H__

#ifdef __cplusplus
   extern "C" {
#endif

    int   tcl3dVec3fCompare (float *v1, float *v2, float delta);
    void  tcl3dVec3fIdentity (float *res);
    void  tcl3dVec3fCopy (float *v, float *res);
    float tcl3dVec3fLength (float *v);
    int   tcl3dVec3fNormalize (float *v);
    float tcl3dVec3fDistance (float *v1,  float *v2);
    float tcl3dVec3fDotProduct (float *v1,  float *v2);
    void  tcl3dVec3fCrossProduct (float *v1, float *v2, float *res);
    void  tcl3dVec3fAdd (float *v1, float *v2, float *res);
    void  tcl3dVec3fSubtract (float *v1, float *v2, float *res);
    void  tcl3dVec3fScale (float scalar, float *v, float *res);
    void  tcl3dVec3fPlaneNormal (float *p1, float *p2, float *p3, float *res);

    int  tcl3dMatfCompare (float *m1, float *m2, float delta);
    void tcl3dMatfIdentity (float *res);
    void tcl3dMatfCopy (float *m, float *res);
    void tcl3dMatfTranslatev (float *v, float *m);
    void tcl3dMatfTranslate (float tx, float ty, float tz, float *m);
    void tcl3dMatfRotate (float angle, float *axis, float *m);
    void tcl3dMatfRotateX (float angle, float *m);
    void tcl3dMatfRotateY (float angle, float *m);
    void tcl3dMatfRotateZ (float angle, float *m);
    void tcl3dMatfScalev (float *v, float *m);
    void tcl3dMatfScale (float sx, float sy, float sz, float *m);
    void tcl3dMatfTransformPoint (float *v, float *m, float *res);
    void tcl3dMatfTransformVector (float *v, float *m, float *res);
    void tcl3dMatfMult (float *m1, float *m2, float *res);
    void tcl3dMatfInvert (float *m, float *res);
    void tcl3dMatfTranspose (float *m, float *res);
    int  tcl3dMatfGetAngles (float *m, float *res);
    int  tcl3dMatfGetAngles1 (float *m, float *res);

    unsigned int tcl3dRoundUpPow2 (unsigned int val);
    unsigned int tcl3dIsPow2 (unsigned int val);

#ifdef __cplusplus
   }
#endif

#endif /* __TCL3D_VECMATH_H__ */
