package require tcl3d

proc CompareMat { m1 m2 } {
    if { ! [tcl3dMatfCompare $m1 $m2 1.0E-6] } {
        puts "Tcl3D Matrix:"
        tcl3dMatfPrint $m1 "%10.7f"
        puts "OpenGL Matrix:"
        tcl3dMatfPrint $m2 "%10.7f"
        error "Matrices are not identical"
    }
}

togl .t
glMatrixMode GL_MODELVIEW

set v     [tcl3dVector GLfloat 16]
set vgl   [tcl3dVector GLfloat 16]
set tfm   [tcl3dVector GLfloat 16]
set tv    [tcl3dVectorFromArgs GLfloat 1 2 3]
set p     [tcl3dVectorFromArgs GLfloat 1 2 3]
set resp  [tcl3dVectorFromArgs GLfloat 10 10 10]
set axis1 [tcl3dVectorFromArgs GLfloat 1 1 1]
set axis2 [tcl3dVectorFromArgs GLfloat 0.3 -0.8 1.6]

puts "New matrix:"
set mi   [tcl3dVector GLfloat 16]
set migl [tcl3dVector GLfloat 16]
CompareMat $mi $migl
tcl3dMatfPrint $mi
puts ""

puts "Identity matrix:"
tcl3dMatfIdentity $mi
glLoadIdentity
glGetFloatv GL_MODELVIEW_MATRIX $migl
CompareMat $mi $migl
tcl3dMatfPrint $mi
puts ""

puts "Copied identity matrix:"
tcl3dMatfCopy $mi $v
glPushMatrix
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Translation matrix (created with tcl3dMatfTranslate):"
tcl3dMatfTranslate 1 2 3 $v
glTranslatef 1 2 3
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Translation matrix (created with tcl3dMatfTranslatev):"
tcl3dMatfCopy $mi $v
tcl3dMatfTranslatev $tv $v
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Rotation matrix (90 degrees around X):"
tcl3dMatfCopy $mi $v
tcl3dMatfRotateX 90 $v
glLoadIdentity
glRotatef 90  1 0 0
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Rotation matrix (90 degrees around Y):"
tcl3dMatfCopy $mi $v
tcl3dMatfRotateY 90 $v
glLoadIdentity
glRotatef 90  0 1 0
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Rotation matrix (90 degrees around Z):"
tcl3dMatfCopy $mi $v
tcl3dMatfRotateZ 90 $v
glLoadIdentity
glRotatef 90  0 0 1
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Rotation matrix (90 degrees around axis (1,1,1)):"
tcl3dMatfCopy $mi $v
tcl3dMatfRotate 90 $axis1 $v
glLoadIdentity
glRotatef 90  1 1 1
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

set axis2 [tcl3dVectorFromArgs GLfloat 0.3 -0.8 1.6]
puts "Rotation matrix (32.2 degrees around axis (0.3,-0.8,1.6)):"
tcl3dMatfCopy $mi $v
tcl3dMatfRotate 32.2 $axis2 $v
glLoadIdentity
glRotatef 32.2  0.3 -0.8 1.6
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Scale matrix (created with tcl3dMatfScale):"
tcl3dMatfCopy $mi $v
tcl3dMatfScale 1 2 3 $v
glLoadIdentity
glScalef 1 2 3
glGetFloatv GL_MODELVIEW_MATRIX $vgl
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Scale matrix (created with tcl3dMatfScalev):"
tcl3dMatfCopy $mi $v
tcl3dMatfScalev $tv $v
CompareMat $v $vgl
tcl3dMatfPrint $v
puts ""

puts "Translation matrix applied to point (1,2,3)):"
tcl3dMatfCopy $mi $tfm
tcl3dMatfTranslate 3 2 1 $tfm
tcl3dMatfTransformPoint $p $tfm $resp
tcl3dMatfPrint $tfm
puts -nonewline "Transformed point: "
tcl3dVec3fPrint $resp
puts ""

puts "Translation matrix applied to vector (1,2,3)):"
tcl3dMatfCopy $mi $tfm
tcl3dMatfTranslate 3 2 1 $tfm
tcl3dMatfTransformVector $p $tfm $resp
tcl3dMatfPrint $tfm
puts -nonewline "Transformed vector: "
tcl3dVec3fPrint $resp
puts ""

exit
