/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dSDL
 *      Filename:       tcl3dSDL.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    Main SWIG file for wrapping the Simple DirectMedia
 *                      Library (SDL).
 *                      This file is responsible for the Tcl package tcl3dSDL.
 *
 *                      The macro USE_SDL_IF must be defined at compilation
 *                      time to include the functionality of the SDL library
 *                      into the Tcl3D package.
 *
 *****************************************************************************/

// The name of our generated Tcl package.

%module tcl3dSDL


// Code to initialize the Tk stubs library.

%{
#include <tk.h>
%}

%init %{
#ifndef USE_SDL_IF
    Tcl_SetResult(interp, (char *) "SDL library not wrapped", TCL_STATIC);
    return TCL_ERROR;
#endif

#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, (char*)"8.4", 0) == NULL) {
      return TCL_ERROR;
    }
#endif
%}

#ifdef USE_SDL_IF

// Standard modules from the SWIG library

%include typemaps.i


// Common Tcl3D wrapper modules.

%include tcl3dVectors.i
%include tcl3dConstHash.i
%include tcl3dPointer.i
%include tcl3dArgv.i
%include tcl3dTypemaps.i


// Wrap the SDL library.

%include sdl.i

#endif // USE_SDL_IF
